import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;


public class Lanczos{
	public static double[][] q;
	public static boolean verbose = false;
    
	public static void main(String args[]){
		 
        
        List<Matrix> ev = new LinkedList<Matrix>();
        
        double [][] A = {{1,1,3,0,1,1,2},
        				 {1,2,1,1,2,1,0},
        				 {3,1,4,3,0,2,1},
        				 {0,1,3,1,1,0,1},
        				 {1,2,0,1,1,1,0},
        				 {1,1,2,0,1,1,2},
        				 {2,0,1,1,0,2,2}};
        
    	
	    int n = A.length;
	    
    	
        double[][] I = new double[n][n];
        q = new double[n][];
        
    	 double[][] o = (double[][])A.clone();
    	int i = 0;
        for(i = 0; i < n; i++)
	        for(int j = 0; j < n; j++)
	        	I[i][j] = (i == j) ? 1 : 0;

        System.out.println("Starting Matrix");
        printM(A);
        
        int j = 0;
        A = lanczos(A); 
        double[][] l = (double[][])A.clone();
        System.out.println(" ");
        System.out.println("Generating a tri-diagonal matrix");
        printM(A);
        System.out.println(" ");

        ArrayList<double [][]> QRarrays = null;
        boolean iteration= true;
        i = 0;
       
        while(iteration)
        {
        	QRarrays = qRDecompose(A);
            A = matrixMultiplication(QRarrays.get(1),QRarrays.get(0));
            if(checkSubDiagonal(A))
            {
            	iteration = false;
            }
        }
       
        if(verbose)
        {
        	System.out.println(" ");
        	System.out.println("QR factoriztion");
        	printM(A);
        }
        
        //ATTN: don't put .3f in calculating eigen
        double [] eigen = new double[n];
        
        for(i=0; i<n;i++)
        	for(j=0; j <n; j++)
        	{
        		if(i==j)
        		{
        			if(Math.abs(A[i][j])<0.001)  A[i][j]=0;
        			eigen[i] = A[i][j];
        		}
        		
        	}
        
        //sorting
         for(int out=eigen.length-1; out>1; out--)
         {
        	 for(int in=0; in<out; in++)    
        	     if( Math.abs(eigen[in]) < Math.abs(eigen[in+1]))    
        	     {
        	    	 double temp = eigen[in];
        	    	 eigen[in] = eigen[in+1];
        	    	 eigen[in+1] = temp;
        	      }
         }
        
        if(verbose)
        {
        	System.out.println(" ");
        	System.out.println("Eigenvalues sorted");
        	for(i=0; i<n;i++)
        		System.out.print(eigen[i] + "  ");
        }
      
        double x[][] = copy(l);
        for(int k=0; k<n; k++)
        {
        	for(i=0; i<n;i++)
        		for(j=0; j <n; j++)
        		{
        			if(i==j)
        			{
        				x[i][j] =  x[i][j]-eigen[k];
        			}
        		
        		}
        	//System.out.println("");
            Matrix d = new Matrix(x);
            if(verbose)
            {
            	d.show();
            } 
            Matrix fm = d.solve();
  			Matrix fs = fm.getNorm();
  			ev.add(fs);
            x = copy(l);
            if(verbose)
            {
            	printM(l);
            }
        } 
       
        Matrix eigenvectors = getV(ev);
        if(verbose)
        	eigenvectors.show();
        //ATTN: transpose so that q can be in column vector, right now it's horizontal
        Matrix lancvectors = new Matrix(q).transpose();
        lancvectors.getRidOfNegativeZero();
        if(verbose)
        	lancvectors.show();
        System.out.println("Vector U");
        Matrix left = lancvectors.times(eigenvectors.Clone());
        left.show();
        System.out.println("Vector S");
        if(verbose)
        	new Matrix(l).show();
        Matrix singular = getS(eigen);
        singular.show(); 
        System.out.println("Transpose of vector V");
        
        Matrix c = new Matrix(o).transpose().times(left);
        Matrix right = c.getMultiNorm();
        right = getTransposeOfV(right);
       
		right.show();
		
		System.out.println("Multiplication of three matrices above");
		Matrix a = left.times(singular).times(right);
		a.show();
        
    }
    
    
    public static boolean checkSubDiagonal(double [][] A)
    {
    	for(int i=0; i < A.length; i++)
    	{
    		for(int j=i+1; j <A.length; j++)
    		{
    			if(Math.abs(A[j][i]) !=0.0)
    				return false;
    		}
    	}
    	
    	return true;
    }
    
    public static Matrix getTransposeOfV(Matrix V)
	{
		return V.transpose();
	}
    
    public static Matrix getS(double []eigen)
    { 
	   double[][] S = new double[eigen.length][eigen.length];
	   for(int i=0; i < eigen.length; i++)
		{
		   double val = Math.abs(eigen[i]);
		   for(int j=0; j<eigen.length; j++)
			{
				 if(i==j)
				 {	
					 S[i][j]= val;
				 }
			}
		}
	
	   return new Matrix(S);
    }

    public static Matrix getV(List<Matrix> ev)
	{
		ListIterator<Matrix> mitr = ev.listIterator();
		
		Matrix V = new Matrix(ev.size(), ev.size());
		int j=0;
		while(mitr.hasNext())
		{
			Matrix e = mitr.next();
			for(int i=0;i<ev.size();i++)
			{
				V.data[i][j] = e.data[i][0];
			}
			
			j++;
			
		}
		
		return V;
	}
	
    public static ArrayList<double[][]> qRDecompose(double[][] M)
    {
     
        ArrayList<double[][]> QRarrays = new ArrayList<double[][]>();
        int n = M.length; 
        double[][] A = (double[][]) M.clone();
        double[][] Q = new double[n][];
        double[][] R = new double[n][n];

        int i = 0;
        int j = 1;
        while(i<n)
        {
        	R[i][i] = Math.sqrt(dot(A[i],A[i]));
            Q[i] = vectorScalarMultiplication(A[i],1/R[i][i]);
            while(j<n)
            {
            	R[j][i] = dot(Q[i],A[j]);
                A[j] = vectorMinus(A[j],vectorScalarMultiplication(Q[i],R[j][i]));
                j++;

             }
             i++;
             j=i+1;
        }

        QRarrays.add(Q);
        QRarrays.add(R);
        return QRarrays;
    }


    public static double[][] lanczos(double[][] M)
	{
		int n = M.length;
	    double [][] temp = new double[1][n];
        double[] b = new double[n+1];
        double[] a = new double[n+1];
        double[][] I = new double[n][n];
        double[] zeroVector = zerorize(n);
	    double[] columnVector;
	    double[][] lanczosM = new double[n][];
	   
	    int i = 0;
        for(i = 0; i < n; i++)
	        for(int j = 0; j < n; j++)
	        	I[i][j] = (i == j) ? 1 : 0;
        
        //considering index 0 as 1
        i=0;
        b[0] = 0;
        q[0] = zerorize(n);
        q[0][0] = 1; 
        
        a[0] = dot(q[0],matrixVectorMultiplication(M,q[0]));
        temp[0] = vectorMinus(matrixVectorMultiplication(M,q[0]), vectorScalarMultiplication(q[0],a[0]));
         double [] temp2 = copyV(temp[0]);
        for(int k=0; k <=i ;k++)
  		  temp[0] =vectorMinus(temp[0],vectorScalarMultiplication(q[k],dot(temp2,q[k])));
        b[0] = Math.sqrt(dot(temp[0],temp[0]));
        q[1] = vectorScalarMultiplication(temp[0],1/b[0]);
        i++;
        
        
        while(i<n-1) // if n, index out of bound
	    {
	    	a[i] = dot(q[i],matrixVectorMultiplication(M,q[i]));
	        temp[0] = vectorMinus(vectorMinus(matrixVectorMultiplication(M,q[i]), vectorScalarMultiplication(q[i-1],b[i-1])) ,vectorScalarMultiplication(q[i], a[i]));  
	        double [] temp1 = copyV(temp[0]);
	    	for(int k=0; k <=i ;k++)
	    	{	  
	    		temp[0] =vectorMinus(temp[0],vectorScalarMultiplication(q[k],dot(temp1,q[k])));
	    
	    	}
			b[i] = Math.sqrt(dot(temp[0],temp[0]));
	        q[i+1] = vectorScalarMultiplication(temp[0],1/b[i]);
	        i++;
	     }
	    
	    a[i] = dot(q[i],matrixVectorMultiplication(M,q[i]));
	       
	    //constructing return
	    //initial
	    i = 0;
	    int k=0; 
	    columnVector = (double[]) zeroVector.clone();
	    columnVector[i] = a[k];
	    columnVector[i+1] = b[k];
	    lanczosM[i] = columnVector;
	        
	    //mid
	    i++;
	    k++;
	    while(i<n-1) 
	    {
	    	columnVector = (double[]) zeroVector.clone();
	        columnVector[i-1] = b[k-1];
	        columnVector[i] = a[k];
	        columnVector[i+1] = b[k];
	        lanczosM[i] = columnVector;
	        i++;
	        k++;
	     }
	        
	    //end
	    columnVector = (double[]) zeroVector.clone();
	    columnVector[i-1] = b[k-1];
	    columnVector[i] = a[k];
	    lanczosM[i] = columnVector;
	    
	    return lanczosM;
	}
    
    public static double[][] copy(double [][] temp) 
	{
	
		double [][] m = new double[temp.length][temp[0].length];
		for (int i = 0; i < temp.length; i++) 
		{
	      for (int j = 0; j < temp[0].length; j++) 
	      {
	    	  m[i][j] = temp[i][j];
	      } 
	    } 
		return m;
	}

   
    public static double dot(double[] u, double[] v)
    {
        int n = v.length;
        double sum = 0;
        for(int i=0; i<n; i++)
        	sum = sum + u[i]*v[i];
        
        return sum;
    }

    public static double[][] transpose(double[][] M)
    {
        int n = M.length;
        int m = M[0].length;
        double[][] A = new double[n][m];
        
        for (int i = 0; i < n; i++)
            for (int j = 0; j < m; j++)
                A[j][i] = M[i][j];
        return A;
     }

    public static double[] vectorPlus(double[] u, double[] v)
    {
        int n = u.length;
        double[] A = new double[n];
        for(int i=0; i<n; i++)
        {
        	A[i] = u[i] + v[i];
            i++;
        }
        return A;
    }

    public static double[][]  matrixMinus(double[][] M, double[][] N)
    {
        int n = M.length; 
        double[][] A = new double[n][];
        for(int i=0; i<n; i++)
        {
            A[i] = vectorMinus(M[i],N[i]);
        }
        return A;
    }

    public static double[] vectorMinus(double[] u, double[] v)
    {
        int n = u.length;
        double[] A = new double[n];
        for(int i=0; i<n; i++)
        {
        	A[i] = u[i] - v[i];
            if (A[i] == -0.0)  
        	 	A[i]= 0.0;
        }
        return A;
    }

    public static double[] vectorScalarMultiplication(double[] v, double k)
    {
        int n = v.length; 
        double[] A = new double[n];
        for(int i=0; i<n; i++)
        {
            A[i] = v[i]*k;
           
        }
        return A;
    }
	
	 public static double [] copyV(double [] temp)
    {
    	double [] m = new double[temp.length];
		for (int i = 0; i < temp.length; i++) 
		{
			m[i] = temp[i];
	    } 
		return m;
    }
	
    public static double[][] matrixMultiplication(double[][] M, double[][] N)
    {
        int n = M.length; 
        int m = N.length;
        double[][] A = new double[m][];
        for(int i=0; i<n; i++)
        {
        	A[i] = matrixVectorMultiplication(M,N[i]);
       
        }
        return A;
    }

    public static double[] matrixVectorMultiplication(double[][] M, double[] v)
    {
        int n = M.length; 
        int m = M[0].length;
        double[] x = new double[n];
      
        for(int i=0; i<m; i++)
        {
        	for(int j=0; j <n; j++)
        	{
        		x[i] = x[i]+M[j][i]*v[j];
        		if (Double.parseDouble(String.format("%.3f",x[i])) == -0.000f)  
        			x[i]= 0.000f;
        	}
        	
        }
        return x;
    }

   
    public static double[] zerorize(int n)
	{
		double[] z = new double[n];
	    for(int i = 0; i<n;i++)
	    	z[i] = 0;
	    return z;
	}

    public static void printM(double[][] M)
	{
    	 for(int i=0; i<M.length; i++)
         {
         	for(int j=0; j <M[0].length; j++)
         	{
	    		System.out.print(String.format("%.3f",M[i][j]) + " ");
	        }
	        
	    	System.out.println("  ");
	     }
	}
}


