public class Matrix {
    public int M;             // number of rows
    public int N;             // number of columns
    public double[][] data;   


    public Matrix(int M, int N) {
        this.M = M;
        this.N = N;
        data = new double[M][N];
    }
    
    public double getElement( int m, int n )
    {
       return data[m][n];
    }
	
    public int getRowNum()
    {
    	return M;
    }
  
    public Matrix zeroize()
    {
    	Matrix z = new Matrix(M, N);
    	for (int i = 0; i < M-1; i++)
    		for(int j=0; j <N; j++)
    			z.data[i][j] = 0.0;
      
    	z.data[M-1][0] =0;
    	return z;
    }

	public Matrix specialClone()
    {
    	Matrix m = new Matrix(M,N);
    	for (int i = 0; i < M; i++) 
            for (int j = 0; j < N; j++) 
				m.data[i][j]= this.data[i][j];
    
		m.data[M-1][N-1]=1;
		return m;
	}
  
	public Matrix Clone()
	{
		Matrix m = new Matrix(M,N);
	    for (int i = 0; i < M; i++) 
	    	for (int j = 0; j < N; j++) 
	    		m.data[i][j]= this.data[i][j];
		return m;
	}
	  
	public void getRidOfNegativeZero()
	{
		for (int i = 0; i < M; i++) 
			for (int j = 0; j < N; j++) 
	        {	
				if(Double.parseDouble(String.format("%.3f",this.data[i][j])) == -0.000f)
					this.data[i][j]=0.000f;
	        }
	  }
	  
  
	public int getColNum()
	{
		return N;
	}
  
	public double norm()
	{
		double norm=0.0;
  
		for(int i=0; i<M; i++)
			norm = norm + Math.pow( data[i][0], 2.0);
		return Math.sqrt(norm);
	} 
  
	public double multinorm(int c)
	{
		double norm=0.0;
		  
	    for(int i=0; i<M; i++)
	        norm = norm + Math.pow( data[i][c], 2.0);
	    return Math.sqrt(norm);
	}
  
	public Matrix getMultiNorm()
	{
		Matrix m = new Matrix(M,N);
		
		for(int i=0; i<M; i++)
		{
			for(int j=0; j < N; j++)
			{
				double n = multinorm(j);
				if(n!=0)
					m.data[i][j] = this.data[i][j]/n;
				else
					m.data[i][j]=0.0;
			}
		}
	
		return m;
	
	}
	

    // create matrix based on 2d array
    public Matrix(double[][] data) 
    {
        M = data.length;
        N = data[0].length;
        this.data = new double[M][N];
        for (int i = 0; i < M; i++)
            for (int j = 0; j < N; j++)
                    this.data[i][j] = data[i][j];
    }
    
   
   public Matrix transpose()
   {
        Matrix A = new Matrix(N, M);
        for (int i = 0; i < M; i++)
            for (int j = 0; j < N; j++)
                A.data[j][i] = this.data[i][j];
        return A;
    }

   
    public Matrix plus(Matrix B)
    {
        Matrix A = this;
        if (B.M != A.M || B.N != A.N) throw new RuntimeException("Illegal matrix dimensions.");
        Matrix C = new Matrix(M, N);
        for (int i = 0; i < M; i++)
            for (int j = 0; j < N; j++)
                C.data[i][j] = A.data[i][j] + B.data[i][j];
        return C;
    }


   public Matrix minus(Matrix B)
   {
        Matrix A = this;
        if (B.M != A.M || B.N != A.N) throw new RuntimeException("Illegal matrix dimensions.");
        Matrix C = new Matrix(M, N);
        for (int i = 0; i < M; i++)
            for (int j = 0; j < N; j++)
                C.data[i][j] = A.data[i][j] - B.data[i][j];
        return C;
    }

    
   public Matrix times(Matrix B) 
   {
        Matrix A = this;
        if (A.N != B.M) throw new RuntimeException("Illegal matrix dimensions.");
        Matrix C = new Matrix(A.M, B.N);
        
        for (int i = 0; i < C.M; i++)
            for (int j = 0; j < C.N; j++)
                for (int k = 0; k < A.N; k++)
                    C.data[i][j] += (A.data[i][k] * B.data[k][j]);
       
        
        for(int i=0; i <C.M; i++)
        	for(int j=0; j<C.M; j++)
        	{		
        		if( Double.parseDouble(String.format("%.3f",C.data[i][j])) == -0.000f)  
        			C.data[i][j]=0.0;
        	}
        return C;
    }
   
    public Matrix solve() 
    {
    	int i=0;
        int j=0;     
        
    	Matrix A = new Matrix(M, N);
    	for ( i = 0; i < M; i++)
    		for(j=0; j <N;j++)
    			A.data[i][j] = data[i][j];
    	
    	//System.out.println(">>> Before computing Gaussian Eliminataion to compute eigenvector");
    	//A.show();
    	
    	int step=0;
        for (int p = 0; p < A.M; p++) 
        {

           //System.out.println(">>> Step "+ step+ " for Gaussian Elimination.");
           //A.show();
           int max = p;
           for (i = p + 1; i < A.M; i++) 
           {
               if (Math.abs(A.data[i][p]) > Math.abs(A.data[max][p]))
               {
                   max = i;
               }
           }
           double[] temp = A.data[p]; 
           A.data[p] = A.data[max]; 
           A.data[max] = temp;
        
           
            for ( i = p + 1; i < A.M; i++) 
            {
            	double alpha;
            	
            	if(A.data[i][p]==0)
            	   continue;
            	else
            		alpha = A.data[p][p]/A.data[i][p];
               
                for (j = p; j < N; j++) 
                {
                    A.data[i][j] = A.data[p][j] - (alpha * A.data[i][j]);
                }
            }
            
            step++;
           
        } // end outer for
   
      
    	Matrix x = new Matrix(N, 1);
        for ( i = 0; i < M; i++)
        	x.data[i][0] = 0.0;
        x.data[N-1][0] = 1;
        
        for( i=A.M-2;i>=0;i--)
        {
             
            double sum=0;
            for(j=(i+1);j<A.M;j++)
            {
            	 sum += (A.data[i][j] * x.data[j][0]);
            }
            //System.out.println(sum);

        	double temp = x.data[i][0];
            if(A.data[i][i]!=0)
            {	
            	x.data[i][0] = (temp-sum) /A.data[i][i];
            }
            else
            	x.data[i][0]=1;
         }
    	
  
        //System.out.println(">>> After computing back-substitution for x,y,z values");
        //x.show();
        //System.out.println("");
        return x;
    }
  
    
    public double format(double a)
    {
    	 if (a > -0.001 && a < 0.001)
    		 return 0;
    	 else
    		 return a;
    }
    
    
	public void show() {
        for (int i = 0; i < M; i++) {
            for (int j = 0; j < N; j++) 
                System.out.printf("%9.3f ", data[i][j]);
            System.out.println();
        }
    }

	public Matrix getNorm()
	{
		Matrix m = new Matrix(M,1);
		double n = norm();
		for(int i=0; i<M; i++)
		{
			m.data[i][0] = this.data[i][0]/n;
		}
	
		return m;
	}
	
}

